html1 = """<li class="clearfix">
<span class="interface nearby_stations_schools_national_rail_station" title="London Bridge"></span>
<span class="nearby_stations_schools_name" title="London Bridge">London Bridge</span>


            (0.4 miles)

        </li>"""


html2 = """<li class="clearfix">
<span class="interface nearby_stations_schools_london_underground_station" title="Borough"></span>
<span class="nearby_stations_schools_name" title="Borough">Borough</span>


            (1.2 miles)

        </li>"""
